@@ -13,6 +13,11 @@ utils::globalVariables(c("na.pass"))
 #' Note: this package is in its early stages and the interface may change
 #' for future versions.
 #'
+#' @references
+#'
+#' Jones PJ, Mair P, Simon T, Zeileis A (2019). Network Model Trees. OSF
+#' Preprints. https://doi.org/10.31219/osf.io/ha4cw
+#'
 #' @examples
 #' 
 #' set.seed(1)
@@ -47,12 +52,11 @@ networktree <- function(...) UseMethod("networktree")
 
 #' @param nodevars the variables with which to compute the network. Can be vector, matrix, or dataframe
 #' @param splitvars the variables with which to test split the network. Can be vector, matrix, or dataframe
-#' @param type the type of network to compute. Can be "cor", "pcor", or "glasso". Note that networks
-#' are always stored internally as correlation matrices, but will be auto-adjusted in plots etc. according
-#' to type
 #' @param method "mob" or "ctree"
 #' @param model can be any combination of c("correlation", "mean", "variance")
 #' splits are determined based on the specified characteristics
+#' @param transform should stored correlation matrices be transformed to partial correlations
+#' or a graphical lasso for plotting? Can be set to "cor" (default), "pcor", or "glasso"
 #' @param na.action a function which indicates what should happen when the data
 #' contain missing values (\code{NA}s).
 #' @param weights weights
@@ -62,21 +66,36 @@ networktree <- function(...) UseMethod("networktree")
 #'@rdname networktree
 #'@export
 networktree.default <- function(nodevars, splitvars, 
-                                type=c("cor", "pcor", "glasso"),
                                 method=c("mob","ctree"),
                                 model ="correlation",
-                                na.action=na.pass,
+                                transform=c("cor", "pcor", "glasso"),
+                                na.action=na.omit,
                                 weights=NULL,...){
-  nodevars <- as.matrix(nodevars)
-  splitvars <- as.matrix(splitvars)
-  if(is.null(colnames(nodevars))){colnames(nodevars) <- paste('nodevars',1:ncol(nodevars),sep="")}
-  if(is.null(colnames(splitvars))){colnames(splitvars) <- paste('splitvars',1:ncol(splitvars),sep="")}
-  
+  if (!inherits(nodevars, "data.frame")) {
+      if (is.vector(nodevars)) {
+	  nodevars <- matrix(nodevars, ncol = 1)
+      }
+      if (any(is.null(colnames(nodevars)))) {
+          colnames(nodevars) <- paste('nodevars',1:ncol(nodevars),sep="")
+      }
+      nodevars <- as.data.frame(nodevars)
+  }
+
+  if (!inherits(splitvars, "data.frame")) {
+      if (is.vector(splitvars)) {
+	  splitvars <- matrix(splitvars, ncol = 1)
+      }
+      if (any(is.null(colnames(splitvars)))) {
+          colnames(splitvars) <- paste('splitvars',1:ncol(splitvars),sep="")
+      }
+      splitvars <- as.data.frame(splitvars)
+  }
+
   if(method[1]=="mob"){
     d <- data.frame(nodevars,splitvars)
     form <- paste(paste(colnames(nodevars), collapse=" + "), "~",paste(colnames(splitvars), collapse=" + "))
     form <- as.formula(form)
-    res <- networktree.formula(form, data = d, type=type, method=method, na.action=na.action, model = model, ...)
+    res <- networktree.formula(form, data = d, transform=transform, method=method, na.action=na.action, model = model, ...)
     
   } else if(method[1]=="ctree"){
     netdata <- as.data.frame(nodevars); splitvars <- as.data.frame(splitvars)
@@ -88,7 +107,7 @@ networktree.default <- function(nodevars, splitvars,
     tree <- partykit::ctree(formula=f1, data=d,
                             ytrafo=function(data, weights,control) {cortrafo(data=data, weights=weights, control=control, n=n, model=model)},
                              na.action=na.action, control=partykit::ctree_control(...))
-    class(tree) <- c("networktree", "ctree_networktree", type[1], class(tree))
+    class(tree) <- c("networktree", "ctree_networktree", transform[1], class(tree))
     class(tree$info$call) <- model ## discreetly store model
     res <- tree
   }
@@ -105,9 +124,9 @@ networktree.default <- function(nodevars, splitvars,
 #' @param data a data frame containing the variables in the model
 #'@rdname networktree
 #'@export
-networktree.formula <- function(formula, data, type=c("cor", "pcor", "glasso"), 
+networktree.formula <- function(formula, data, transform=c("cor", "pcor", "glasso"), 
                                 method=c("mob","ctree"),
-                                na.action=na.pass, model="correlation", ...)
+                                na.action=na.omit, model="correlation", ...)
 {
   if(method[1]=="mob"){
     
@@ -120,12 +139,11 @@ networktree.formula <- function(formula, data, type=c("cor", "pcor", "glasso"),
 
     ## set default for minsize if not specified
     if(is.null(control$minsize)) { 
-	F <- Formula::Formula(formula)
-
-	k <- ncol(stats::model.matrix(~ 0 + .,
-		 model.part(F, stats::model.frame(F, data = data, rhs = 0), lhs = TRUE)
-	))
-	control$minsize <- k * (k-1) / 2 + 1
+	    F <- Formula::Formula(formula)
+	    k <- ncol(stats::model.matrix(~ 0 + .,
+		              model.part(F, stats::model.frame(F, data = data, rhs = 0), lhs = TRUE)
+	              ))
+	    control$minsize <- 2 * k + k * (k-1) / 2
     }
     
     ## control options for mvnfit
@@ -142,7 +160,7 @@ networktree.formula <- function(formula, data, type=c("cor", "pcor", "glasso"),
     
     ## extend class and keep original call
     rval$info$call <- cl
-    class(rval) <- c("networktree", "mob_networktree", type[1], class(rval))
+    class(rval) <- c("networktree", "mob_networktree", transform[1], class(rval))
     res <- rval
   } else if(method[1]=="ctree"){
     charformulaLHS <-   strsplit(as.character(formula), "+", fixed=T)[[2]]
@@ -152,7 +170,7 @@ networktree.formula <- function(formula, data, type=c("cor", "pcor", "glasso"),
     res <- partykit::ctree(formula=formula, data=data,
                             ytrafo=function(data, weights,control) {cortrafo(data=data, weights=weights, control=control, n=n, model=model)},
                             na.action=na.action, control=partykit::ctree_control(...))
-    class(res) <- c("networktree", "ctree_networktree", type[1], class(res))
+    class(res) <- c("networktree", "ctree_networktree", transform[1], class(res))
     class(res$info$call) <- model ## discreetly store model
   }
   return(res)
@@ -160,10 +178,25 @@ networktree.formula <- function(formula, data, type=c("cor", "pcor", "glasso"),
 
 
 ## methods for networktree class
+
+#' Printing 'treenetwork' objects
+#'
+#' Wraps print.modelparty to print a tree model with networks on the ends. 
+#'
+#' @param x an object of type 'networktree'
+#' @param parameters print parameters for each partition? 
+#' See getnetwork function for extracting parameters conveniently
+#' @param FUN only evaluated if parameters=TRUE, passed to print.modelparty
+#' @param ... additional arguments passed print.modelparty
+#'
+#'@export
 print.networktree<- function(x,
-                             title = "Network tree", objfun = "negative log-likelihood", ...)
+                             parameters=FALSE,
+                             FUN=NULL,
+                             ...)
 {
-  partykit::print.modelparty(x, title = title, objfun = objfun, ...)
+  FUN <- ifelse(!parameters, function(x){""}, FUN)
+  partykit::print.modelparty(x, title = "Network tree object", FUN=FUN, ...)
 }
 
 #' Plotting 'treenetwork' objects
@@ -172,16 +205,19 @@ print.networktree<- function(x,
 #' are plotted with qgraph, and additional arguments are passed there
 #'
 #' @param x an object of type 'networktree'
-#' @param type "cor", "pcor", or "glasso". If set to NULL, type detected from x
+#' @param transform "cor", "pcor", or "glasso". If set to NULL, transform detected from x
+#' @param layout network layout, passed to qgraph. Default "lock" computes spring 
+#' layout for the full sample and applies this to all graphs
+#' @param partyargs additional arguments (list format) passed to \code{partykit::plot.party}
 #' @param ... additional arguments passed qgraph
 #'
 #'@export
-plot.networktree <- function(x, type = NULL, ...) {
+plot.networktree <- function(x, transform = NULL, layout="lock", partyargs=list(), ...) {
   
   dots <- list(...)
-
-  if(is.null(type)) {
-    type <- if("cor" %in% class(x)) {"cor"} else if ("pcor" %in% class(x)) {"pcor"
+  
+  if(is.null(transform)) {
+    transform <- if("cor" %in% class(x)) {"cor"} else if ("pcor" %in% class(x)) {"pcor"
     } else if("glasso" %in% class(x)) {"glasso"
     } else {"glasso";
       warning("Type of network could not be detected, plotting glasso networks")}
@@ -193,19 +229,67 @@ plot.networktree <- function(x, type = NULL, ...) {
     model <- class(x[[1]]$info$call)
   }
   if("variance" %in% model | "mean" %in% model){
-    message("Network plotting not yet implemented for splits by variance and mean")
-    partykit::plot.party(x)
+    warning("Network plotting not yet implemented for splits by variance and mean.\nPlotting partykit summary.")
+    partyargs <- c(partyargs, list(x=x))
+    do.call(what=partykit::plot.party,args=partyargs)
   } else {
+    if(layout[1]=="lock"){
+      layout <- qgraph::qgraph(getnetwork(x,id=1),layout="spring",DoNotPlot=T)$layout
+    }
     ## plotting network (when model == "correlation")
     net_terminal_inner <- function(obj, ...) {
-      net_terminal(obj, type = type, ...)
+      net_terminal(obj, transform = transform,layout = layout, ...)
     }
     class(net_terminal_inner) <- "grapcon_generator"
-    partykit::plot.party(x, terminal_panel = net_terminal_inner, tp_args = dots)
+    needNewPlot <- tryCatch(
+      {
+        par(new=TRUE)
+        FALSE
+      }, 
+      warning=function(cond){
+        return(TRUE)
+      },
+      silent=T
+    )
+    if(needNewPlot){
+      plot.new()
+      partyargs <- c(partyargs, list(x=x, terminal_panel = net_terminal_inner, newpage=FALSE, tp_args = dots))
+      do.call(what=partykit::plot.party,args=partyargs)
+    } else {
+      partyargs <- c(partyargs, list(x=x, terminal_panel = net_terminal_inner, newpage=TRUE, tp_args = dots))
+      do.call(what=partykit::plot.party,args=partyargs)
+    }
   }
 }
 
 
+#' Predict 'networktree' objects
+#'
+#' Wraps predict.party
+#'
+#' @param object a fitted 'networktree'
+#' @param newdata An optional data frame in which to look for variables with
+#'        which to predict.  If omitted, the fitted values are used.
+#' @param type "node", or "parameter". Specifies whether to predict nodes
+#'        (return value is a vector) or parameters (matrix).
+#' @param ... not used
+#'
+#' @method predict networktree
+#'@export
+predict.networktree <- function(object, newdata = NULL,
+				type = c("node", "parameter"), ...) {
+  type <- match.arg(type)
+
+  ## predict node ids
+  node <- partykit::predict.party(object, newdata = newdata)
+  if(identical(type, "node")) {
+    return(node)
+  }
+
+  ## obtain coefs
+  stats::coef(object)[as.character(node), ]
+}
+
 # Package documentation
 # TODO: fix package documentation. causes a conflict w/networktree function documentation
 #       because they have the same name
